# Generated by roxygen2: do not edit by hand

S3method(as.matrix,clubSandwich)
S3method(augmented_model_matrix,default)
S3method(augmented_model_matrix,iv_robust)
S3method(augmented_model_matrix,lm_robust)
S3method(augmented_model_matrix,plm)
S3method(bread,geeglm)
S3method(bread,gls)
S3method(bread,iv_robust)
S3method(bread,lm_robust)
S3method(bread,lme)
S3method(bread,lmerMod)
S3method(bread,mlm)
S3method(bread,mmrm)
S3method(bread,plm)
S3method(bread,rma.mv)
S3method(bread,rma.uni)
S3method(bread,robu)
S3method(coef_CS,default)
S3method(coef_CS,lme)
S3method(coef_CS,lmerMod)
S3method(coef_CS,mlm)
S3method(coef_CS,rma.ls)
S3method(coef_CS,robu)
S3method(model.frame,iv_robust)
S3method(model.frame,lm_robust)
S3method(model_matrix,default)
S3method(model_matrix,geeglm)
S3method(model_matrix,glm)
S3method(model_matrix,gls)
S3method(model_matrix,iv_robust)
S3method(model_matrix,ivreg)
S3method(model_matrix,lm_robust)
S3method(model_matrix,lme)
S3method(model_matrix,lmerMod)
S3method(model_matrix,mlm)
S3method(model_matrix,plm)
S3method(model_matrix,rma.ls)
S3method(model_matrix,robu)
S3method(na.action,iv_robust)
S3method(na.action,lm_robust)
S3method(na.action,rma)
S3method(print,Wald_test_clubSandwich)
S3method(print,clubSandwich)
S3method(print,coef_test_clubSandwich)
S3method(print,conf_int_clubSandwich)
S3method(residuals,lm_robust)
S3method(residuals_CS,default)
S3method(residuals_CS,geeglm)
S3method(residuals_CS,glm)
S3method(residuals_CS,iv_robust)
S3method(residuals_CS,lme)
S3method(residuals_CS,lmerMod)
S3method(residuals_CS,mlm)
S3method(residuals_CS,plm)
S3method(residuals_CS,rma)
S3method(residuals_CS,robu)
S3method(targetVariance,default)
S3method(targetVariance,geeglm)
S3method(targetVariance,glm)
S3method(targetVariance,gls)
S3method(targetVariance,lme)
S3method(targetVariance,lmerMod)
S3method(targetVariance,mlm)
S3method(targetVariance,mmrm)
S3method(targetVariance,plm)
S3method(targetVariance,rma.mv)
S3method(targetVariance,rma.uni)
S3method(targetVariance,robu)
S3method(v_scale,default)
S3method(v_scale,geeglm)
S3method(v_scale,glm)
S3method(v_scale,lm)
S3method(v_scale,lme)
S3method(v_scale,lmerMod)
S3method(v_scale,mlm)
S3method(v_scale,mmrm)
S3method(v_scale,plm)
S3method(v_scale,rma.mv)
S3method(v_scale,robu)
S3method(vcovCR,default)
S3method(vcovCR,geeglm)
S3method(vcovCR,glm)
S3method(vcovCR,gls)
S3method(vcovCR,iv_robust)
S3method(vcovCR,ivreg)
S3method(vcovCR,lm)
S3method(vcovCR,lm_robust)
S3method(vcovCR,lme)
S3method(vcovCR,lmerMod)
S3method(vcovCR,mlm)
S3method(vcovCR,mmrm)
S3method(vcovCR,plm)
S3method(vcovCR,rma.mv)
S3method(vcovCR,rma.uni)
S3method(vcovCR,robu)
S3method(weightMatrix,default)
S3method(weightMatrix,geeglm)
S3method(weightMatrix,glm)
S3method(weightMatrix,gls)
S3method(weightMatrix,lme)
S3method(weightMatrix,lmerMod)
S3method(weightMatrix,mlm)
S3method(weightMatrix,mmrm)
S3method(weightMatrix,plm)
S3method(weightMatrix,rma.mv)
S3method(weightMatrix,rma.uni)
S3method(weightMatrix,robu)
S3method(weights,robu)
export(Wald_test)
export(coef_test)
export(conf_int)
export(constrain_equal)
export(constrain_pairwise)
export(constrain_zero)
export(findCluster.rma.mv)
export(impute_covariance_matrix)
export(linear_contrast)
export(pattern_covariance_matrix)
export(vcovCR)
import(stats)
importFrom(sandwich,bread)
